from typing import Any, Type, Union, TYPE_CHECKING

from pydantic import BaseModel

from bunnet.exceptions import (
    UnionHasNoRegisteredDocs,
    DocWasNotRegisteredInUnionClass,
)
from bunnet.odm.interfaces.detector import ModelType

if TYPE_CHECKING:
    from bunnet.odm.documents import Document


def parse_obj(
    model: Union[Type[BaseModel], Type["Document"]], data: Any
) -> BaseModel:
    if (
        hasattr(model, "get_model_type")
        and model.get_model_type() == ModelType.UnionDoc
    ):
        if model._document_models is None:
            raise UnionHasNoRegisteredDocs

        if isinstance(data, dict):
            class_name = data["_class_id"]
        else:
            class_name = data._class_id

        if class_name not in model._document_models:
            raise DocWasNotRegisteredInUnionClass
        return parse_obj(model=model._document_models[class_name], data=data)
    if (
        hasattr(model, "get_model_type")
        and model.get_model_type() == ModelType.Document
        and model._inheritance_inited
    ):
        if isinstance(data, dict):
            class_name = data.get("_class_id")
        elif hasattr(data, "_class_id"):
            class_name = data._class_id
        else:
            class_name = None

        if model._children and class_name in model._children:
            return parse_obj(model=model._children[class_name], data=data)

    # if hasattr(model, "_parse_obj_saving_state"):
    #     return model._parse_obj_saving_state(data)  # type: ignore
    return model.parse_obj(data)
